"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const zlib_1 = require("zlib");
const fs_1 = require("../fs");
const Tar_1 = require("../utils/Tar");
const util_1 = require("../utils/util");
const path_1 = require("path");
const scan_1 = require("./scan");
const targzAtomic_1 = require("./targzAtomic");
/**
 * Tar/Gzips a directory or array of files.
 * @function targz
 * @memberof fsn/nextra
 * @param fileName The filename of the archive
 * @param inputFiles The directory or array of filepaths to .tar.gz
 * @param options The options for this .tar.gz
 */
async function targz(fileName, inputFiles, atomic = false) {
    if (atomic)
        return targzAtomic_1.default(fileName, inputFiles);
    if (!Array.isArray(inputFiles))
        inputFiles = [inputFiles];
    const tar = new Tar_1.default(path_1.dirname(inputFiles[0]));
    for (const input of inputFiles) {
        const files = await scan_1.default(input, { filter: stats => stats.isFile() });
        for (const [file, stats] of files)
            tar.append(file, stats);
    }
    return util_1.pipelinePromise(tar, zlib_1.createGzip(), fs_1.createWriteStream(fileName));
}
exports.default = targz;
//# sourceMappingURL=targz.js.map